﻿# Blueking Platform (BK-JOB)'s Code Structure

English | [简体中文](code_framework.md)

```shell script
|- bk-job
  |- docs
  |- scripts
  |- src
    |- backend
    |- frontend
  |- support-files
  |- versionLogs
```
## Project Source Code (src)
The project contains a mixture of vue/java/shell and other languages. They can be divided by function as: frontend, backend, support files, version logs, ops script. and other sub-directories.
### Frontend Code
```shell script
|- bk-job/src
  |- frontend/src
    |- lib   # Third Party Library Source Code
    |- src
      |- common     # Public Module
      |- components # Public Interaction Components
      |- css        # Global CSS
      |- domain   
        |- model            # Service Model
        |- service          # Backend API Service
        |- source           # Backend API Configuration
        |- variable-object  # The Variable Object That Serves Frontend Logic
      |- i18n       # Global Internationalization
      |- images     # Static Images
      |- router     # Routing Configuration
      |- store      # State Management
      |- utils      # Public Method
      |- views      # System Module
        |- account-manage          # Account Management
          |- index.vue                # Module Entrance
          |- local.js                 # Module Internationalization
          |- routes.js                # Module Routing Table Configuration
        |- cron-job                   # Scheduled Task
        |- detect-records          # Dangerous Rule Interception Record
        |- dashboard               # Operation Analysis
        |- detect-records          # Dangerous Command Interception Record
        |- executive-history       # Task Execution History
        |- fast-execution          # Quick Execution
        |- file-manage             # File Management
        |- home                    # Service Overview
        |- notify-manage           # Notification
        |- plan-manage             # Execution Plan Management
        |- public-script-manage    # Public Script Management
        |- script-manage           # Script Management
        |- script-template         # Script Template
        |- service-state           # Service Status
        |- setting                 # Global Settings
        |- task-manage             # Job Management
        |- ticket-manage           # Credential Management
        |- white-ip                # IP White List
        |- 404.vue                 # Routing 404 Page
        |- business-permission.vue # No Service Permission Page
        |- index.vue               # System Module Entrance File
      |- App.vue            # Page Render Entrance
      |- iframe-app.vue     # Rendering Entrance when Accessing via iframe
      |- layout-new.vue     # Navigation Layout
      |- main.js            # Frontend Entrance File
    |- index-dev.html    # Local Dev Service Entrance
    |- index.html        # Build Service Entrance
    |- webpack.config.js # Webpack Config
```

### Backend Microservice Code (Backend)
```shell script
|- bk-job/src
  |- backend
    |- buildSrc     # Custom Gradle Task that enables special features when packaging.
    |- commons      # Public Module
      |- cmdb-sdk   # Refers to the public code of Blueking Management Platform (CMDB)
      |- cmdb-sdk-ext  # Adds Redis throttling to CMDB-SDK.
      |- common        # Universal constants, errors, and other public codes
      |- common-i18n   # Internationalization
      |- common-iam    # Connects to Blueking permission center.
      |- common-redis  # Redis operations
      |- common-security   # Security control
      |- common-spring-ext # General Spring expansion (custom listener, processor, etc.)
      |- common-statistics # General Statistics
      |- common-utils  # General Tools
      |- common-web    # Web tools (filter, interceptor, etc.)
      |- esb-sdk    # Connects to Blueking ESB
      |- gse-sdk    # Connects to Blueking GSE
      |- paas-sdk   # Connects to Blueking PaaS Platform
    |- job-analysis # Statistical Analysis Microservice
      |- api-job-analysis     # API Difinition Abstraction Layer
      |- boot-job-analysis    # Launching and Other Configurations
      |- model-job-analysis   # Table Structure Models Automatically Generated by JOOQ
      |- service-job-analysis # Service Logic Implementation Layer
    |- job-backup   # Backup Management Microservice
    |- job-config   # Configuration Center Microservice 
    |- job-crontab  # Scheduled Task Microservice
    |- job-execute  # Job Execution Microservice
    |- job-file-gateway     # File Gateway Microservice
    |- job-file-worker      # Realization of Access Points
    |- job-file-worker-sdk  # Access Points’ Common Logic SDK
    |- job-gateway  # Microservice Gateway
    |- job-logsvr   # Log Management Microservice
    |- job-manage   # Job Management Microservice
    |- job-ticket   # Ticket Management Microservice
    |- upgrader     # Upgrade Facility Tools
```

### Support Files (support-files)
```shell script
|- bk-job/support-files
  |- bkiam     # Migration File of Permission Changes
  |- sql       # Migration File of Records MySQL Database Changes
  |- templates # Configuration templates that will be replaced with environment variables when storing microservices.
```

### Version Logs (versionLogs)
Stores version logs in both Chinese and English, as well as scripts generated by frontend resources.

### Ops Script (scripts)
Stores the ops scripts of the launch/shut down/restart of various microservices on the backend.
